/*    patchlevel.h
 *
 *    Copyright (C) 1993, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
 *    2003, 2004, 2005, 2006, 2007, 2008, 2009, by Larry Wall and others
 *
 *    You may distribute under the terms of either the GNU General Public
 *    License or the Artistic License, as specified in the README file.
 *
 */

#ifndef __PATCHLEVEL_H_INCLUDED__

/* do not adjust the whitespace! Configure expects the numbers to be
 * exactly on the third column */

#define PERL_REVISION	5		/* age */
#define PERL_VERSION	24		/* epoch */
#define PERL_SUBVERSION	0		/* generation */

/* The following numbers describe the earliest compatible version of
   Perl ("compatibility" here being defined as sufficient binary/API
   compatibility to run XS code built with the older version).
   Normally this should not change across maintenance releases.

   Note that this only refers to an out-of-the-box build.  Many non-default
   options such as usemultiplicity tend to break binary compatibility
   more often.

   This is used by Configure et al to figure out
   PERL_INC_VERSION_LIST, which lists version libraries
   to include in @INC.  See INSTALL for how this works.

   Porting/bump-perl-version will automatically set these to the version of perl
   to be released for blead releases, and to 5.X.0 for maint releases. Manually
   changing them should not be necessary.
*/
#define PERL_API_REVISION	5
#define PERL_API_VERSION	24
#define PERL_API_SUBVERSION	0
/*
   XXX Note:  The selection of non-default Configure options, such
   as -Duselonglong may invalidate these settings.  Currently, Configure
   does not adequately test for this.   A.D.  Jan 13, 2000
*/

#define __PATCHLEVEL_H_INCLUDED__
#endif

/*
	local_patches -- list of locally applied less-than-subversion patches.
	If you're distributing such a patch, please give it a name and a
	one-line description, placed just before the last NULL in the array
	below.  If your patch fixes a bug in the perlbug database, please
	mention the bugid.  If your patch *IS* dependent on a prior patch,
	please place your applied patch line after its dependencies. This
	will help tracking of patch dependencies.

	Please either use 'diff --unified=0' if your diff supports
	that or edit the hunk of the diff output which adds your patch
	to this list, to remove context lines which would give patch
	problems. For instance, if the original context diff is

	   *** patchlevel.h.orig	<date here>
	   --- patchlevel.h	<date here>
	   *** 38,43 ***
	   --- 38,44 ---
	     	,"FOO1235 - some patch"
	     	,"BAR3141 - another patch"
	     	,"BAZ2718 - and another patch"
	   + 	,"MINE001 - my new patch"
	     	,NULL
	     };

	please change it to
	   *** patchlevel.h.orig	<date here>
	   --- patchlevel.h	<date here>
	   *** 41,43 ***
	   --- 41,44 ---
	   + 	,"MINE001 - my new patch"
	     	,NULL
	     };

	(Note changes to line numbers as well as removal of context lines.)
	This will prevent patch from choking if someone has previously
	applied different patches than you.

        History has shown that nobody distributes patches that also
        modify patchlevel.h. Do it yourself. The following perl
        program can be used to add a comment to patchlevel.h:

#!perl
die "Usage: perl -x patchlevel.h comment ..." unless @ARGV;
open PLIN, "patchlevel.h" or die "Couldn't open patchlevel.h : $!";
open PLOUT, ">patchlevel.new" or die "Couldn't write on patchlevel.new : $!";
my $seen=0;
while (<PLIN>) {
    if (/\t,NULL/ and $seen) {
       while (my $c = shift @ARGV){
	    $c =~ s|\\|\\\\|g;
	    $c =~ s|"|\\"|g;
            print PLOUT qq{\t,"$c"\n};
       }
    }
    $seen++ if /local_patches\[\]/;
    print PLOUT;
}
close PLOUT or die "Couldn't close filehandle writing to patchlevel.new : $!";
close PLIN or die "Couldn't close filehandle reading from patchlevel.h : $!";
close DATA; # needed to allow unlink to work win32.
unlink "patchlevel.bak" or warn "Couldn't unlink patchlevel.bak : $!"
  if -e "patchlevel.bak";
rename "patchlevel.h", "patchlevel.bak" or
  die "Couldn't rename patchlevel.h to patchlevel.bak : $!";
rename "patchlevel.new", "patchlevel.h" or
  die "Couldn't rename patchlevel.new to patchlevel.h : $!";
__END__

Please keep empty lines below so that context diffs of this file do
not ever collect the lines belonging to local_patches() into the same
hunk.

 */

#if !defined(PERL_PATCHLEVEL_H_IMPLICIT) && !defined(LOCAL_PATCH_COUNT)
#  if defined(PERL_IS_MINIPERL)
#    define PERL_PATCHNUM "UNKNOWN-miniperl"
#    define PERL_GIT_UNPUSHED_COMMITS /*leave-this-comment*/
#  elif defined(PERL_MICRO)
#    define PERL_PATCHNUM "UNKNOWN-microperl"
#    define PERL_GIT_UNPUSHED_COMMITS /*leave-this-comment*/
#  else
#include "git_version.h"
#  endif
static const char * const local_patches[] = {
	NULL
#ifdef PERL_GIT_UNCOMMITTED_CHANGES
	,"uncommitted-changes"
#endif
	PERL_GIT_UNPUSHED_COMMITS    	/* do not remove this line */
	,"Fedora Patch1: Removes date check, Fedora/RHEL specific"
	,"Fedora Patch3: support for libdir64"
	,"Fedora Patch4: use libresolv instead of libbind"
	,"Fedora Patch5: USE_MM_LD_RUN_PATH"
	,"Fedora Patch6: Provide MM::maybe_command independently (bug #1129443)"
	,"Fedora Patch7: Dont run one io test due to random builder failures"
	,"Fedora Patch15: Define SONAME for libperl.so"
	,"Fedora Patch16: Install libperl.so to -Dshrpdir value"
	,"Fedora Patch22: Document Math::BigInt::CalcEmu requires Math::BigInt (CPAN RT#85015)"
	,"Fedora Patch26: Make *DBM_File desctructors thread-safe (RT#61912)"
	,"Fedora Patch27: Make PadlistNAMES() lvalue again (CPAN RT#101063)"
	,"Fedora Patch28: Make magic vtable writable as a work-around for Coro (CPAN RT#101063)"
	,"Fedora Patch30: Replace EU::MakeMaker dependency with EU::MM::Utils in IPC::Cmd (bug #1129443)"
	,"Fedora Patch31: Fix a memory leak in compiling a POSIX class (RT#128313)"
	,"Fedora Patch32: Do not mangle errno from failed socket calls (RT#128316)"
	,"Fedora Patch33: Fix compiling regular expressions like /\\X*(?0)/ (RT#128109)"
	,"Fedora Patch34: Do not use unitialized memory in $h{\\const} warnings (RT#128189)"
	,"Fedora Patch35: Fix precedence in hv_ename_delete (RT#128086)"
	,"Fedora Patch36: Do not treat %: as a stash (RT#128238)"
	,"Fedora Patch37: Do not crash when inserting a non-stash into a stash (RT#128238)"
	,"Fedora Patch38: Fix line numbers with perl -x (RT#128508)"
	,"Fedora Patch39: Do not let XSLoader load relative paths (CVE-2016-6185)"
	,"Fedora Patch40: Fix a crash when vivifying a stub in a deleted package (RT#128532)"
	,"Fedora Patch41: Fix a crash in \"Subroutine redefined\" warning (RT#128257)"
	,"Fedora Patch42: Fix a crash in lexical scope warnings (RT#128597)"
	,"Fedora Patch43: Fix handling \\N{} in tr for characters in range 128--255 (RT#128734)"
	,"Fedora Patch44: Avoid loading of modules from current directory (CVE-2016-1238)"
	,"RHEL Patch45: Fix CVE-2018-6798 (heap read overflow in regexec.c) (RT#132063)"
	,"RHEL Patch48: Fix CVE-2018-6797 (heap write overflow in regcomp.c) (RT#132227)"
	,"RHEL Patch49: Fix CVE-2018-18311 Integer overflow leading to buffer overflow"
	,"RHEL Patch50: Fix CVE-2018-18314 Heap-buffer-overflow write in regcomp.c"
	,"RHEL Patch51: Fix CVE-2018-18312 Heap-buffer-overflow write in regcomp.c"
	,"RHEL Patch52: Fix CVE-2018-18313 Heap-buffer-overflow read in regcomp.c"
	,"Fedora Patch200: Link XS modules to libperl.so with EU::CBuilder on Linux"
	,"Fedora Patch201: Link XS modules to libperl.so with EU::MM on Linux"
	,NULL
};



/* Initial space prevents this variable from being inserted in config.sh  */
#  define	LOCAL_PATCH_COUNT	\
	((int)(C_ARRAY_LENGTH(local_patches)-2))

/* the old terms of reference, add them only when explicitly included */
#define PATCHLEVEL		PERL_VERSION
#undef  SUBVERSION		/* OS/390 has a SUBVERSION in a system header */
#define SUBVERSION		PERL_SUBVERSION
#endif
